use std::error::Error;

use objects::{YoutubePlaylist, YoutubeVideo};

pub mod objects;

const YOUTUBE_API_URL: &str = "https://www.youtube.com/youtubei/v1/browse?prettyPrint=false";

/*#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    println!("Hello, world!!");
    let rid = get_channel("".to_string()).await.unwrap();
    println!("rid: {}", rid);
    for playlist in &get_playlists("".to_string(), rid).await.unwrap() {
		println!("Playlist: {}", playlist.title);
		let videos = get_playlist(playlist.browse_id.clone()).await.unwrap();
		println!("{:?}", videos);
	}
    Ok(())
}*/

pub async fn get_playlist(browse_id: String) -> Result<Vec<YoutubeVideo>, Box<dyn Error>> {
	let client = reqwest::Client::new();

	let d= r#"{
		"context": {
		  "client": {
			"clientName": "WEB",
			"clientVersion": "2.20201210.01.00",
			"originalUrl": "https://www.youtube.com/",
			"platform": "DESKTOP",
			"clientFormFactor": "UNKNOWN_FORM_FACTOR",
			"newVisitorCookie": true
		  }
		},
		"browseId": "browse_id",
	   }"#;

	let d= d.replace("browse_id", &browse_id);
	

	let resp = client.post(YOUTUBE_API_URL)
	   .body(d)
	   .send()
	   .await?
	   .text()
	   .await?;

	let mut videos = Vec::new();

   	let resp = json::parse(&resp).unwrap();

	if let json::JsonValue::Array(resp) = &resp["contents"]["twoColumnBrowseResultsRenderer"]["tabs"] {
		let tab = &resp[0];
		if let json::JsonValue::Array(contents) = &tab["tabRenderer"]["content"]["sectionListRenderer"]["contents"][0]["itemSectionRenderer"]["contents"][0]["playlistVideoListRenderer"]["contents"] {
			for video in contents {
				let video = &video["playlistVideoRenderer"];
				let mut yt = YoutubeVideo::default();
				yt.videoId = video["videoId"].to_string();
				yt.title = video["title"]["runs"][0]["text"].to_string();
				yt.publisher = video["shortBylineText"]["runs"][0]["text"].to_string();
				yt.lengthSeconds = video["lengthSeconds"].to_string().parse().unwrap();
				videos.push(yt);
			}
		}
	}

	Ok(videos)
}

pub async fn get_playlists(channel_id: String, request_id: String) -> Result<Vec<YoutubePlaylist>, Box<dyn Error>> {
    let client = reqwest::Client::new();

    let d= r#"{
         "context": {
           "client": {
             "clientName": "WEB",
             "clientVersion": "2.20201210.01.00",
             "originalUrl": "https://www.youtube.com/",
             "platform": "DESKTOP",
             "clientFormFactor": "UNKNOWN_FORM_FACTOR",
             "newVisitorCookie": true
           }
         },
         "browseId": "channel_id",
         "params": "request_id"
        }"#;
	
	let d = d.replace("channel_id", &channel_id);
	let d = d.replace("request_id", &request_id);

    let resp = client.post(YOUTUBE_API_URL)
        .body(d)
        .send()
        .await?
        .text()
        .await?;

    let resp = json::parse(&resp).unwrap();

	let mut playlists = Vec::new();

    if let json::JsonValue::Array(resp) = &resp["contents"]["twoColumnBrowseResultsRenderer"]["tabs"] {
      for tab in resp {
        if let json::JsonValue::Short(title) = &tab["tabRenderer"]["title"] {
          if title.as_str() == "Playlists" {
            if let json::JsonValue::Array(contents) = &tab["tabRenderer"]["content"]["sectionListRenderer"]["contents"] {
              if let json::JsonValue::Array(columns) = &contents[0]["itemSectionRenderer"]["contents"] {
				for column in columns {
					if let json::JsonValue::Array(items) = &column["gridRenderer"]["items"] {
						for item in items {
							let item = &item["lockupViewModel"]["metadata"]["lockupMetadataViewModel"];
							let title = item["title"]["content"].as_str().unwrap();
							let browse_id = item["metadata"]["contentMetadataViewModel"]["metadataRows"][1]["metadataParts"][0]["text"]["commandRuns"][0]["onTap"]["innertubeCommand"]["browseEndpoint"]["browseId"].as_str().unwrap();
							let pl_url = item["metadata"]["contentMetadataViewModel"]["metadataRows"][1]["metadataParts"][0]["text"]["commandRuns"][0]["onTap"]["innertubeCommand"]["commandMetadata"]["webCommandMetadata"]["url"].as_str().unwrap();
							playlists.push(YoutubePlaylist{ title: title.to_string(), browse_id: browse_id.to_string(), pl_url: pl_url.to_string()});
						}
					}
				}
              }
            }
          }
        }
      }
    }
    Ok(playlists)
}

pub async fn get_channel(channel_id: String) -> Result<String, Box<dyn Error>>{
    let client = reqwest::Client::new();

	let d = r#"{
         "context": {
           "client": {
             "clientName": "WEB",
             "clientVersion": "2.20201210.01.00",
             "originalUrl": "https://www.youtube.com/",
             "platform": "DESKTOP",
             "clientFormFactor": "UNKNOWN_FORM_FACTOR",
             "newVisitorCookie": true
           }
         },
         "browseId": "browse_id",
         "params": "EglwbGF5bGlzdHPyBgQKAkIA"
        }"#;
	
	let d = d.replace("browse_id", &channel_id);

    let resp = client.post(YOUTUBE_API_URL)
        .body(d)
        .send()
        .await?
        .text()
        .await?;

    let resp = json::parse(&resp).unwrap();

    if let json::JsonValue::Array(resp) = &resp["contents"]["twoColumnBrowseResultsRenderer"]["tabs"] {
      for tab in resp {
        if let json::JsonValue::Short(title) = &tab["tabRenderer"]["title"] {
          if title.as_str() == "Playlists" {
            if let json::JsonValue::Short(params) = &tab["tabRenderer"]["endpoint"]["browseEndpoint"]["params"] {
              return Ok(params.to_string());
            }
          }
        }
      }
    }

    Ok("".to_string())
}